@@ -13,7 +13,6 @@ import sys, os, argparse, json
 from boozetools.macroparse.compiler import compile_string
 from boozetools.macroparse.grammar import DefinitionError
 from boozetools.macroparse import compaction
-from boozetools.parsing.all_methods import PARSE_TABLE_METHODS
 
 def parse_arguments():
 	parser = argparse.ArgumentParser(prog='py -m boozetools', description=__doc__,)
@@ -25,7 +24,6 @@ def parse_arguments():
 	parser.add_argument('--csv', action='store_true', help='Generate CSV versions of uncompressed tables, suitable for inspection.')
 	parser.add_argument('--dev', action='store_true', help='Operate in "development mode" -- which changes from time to time.')
 	parser.add_argument('--dot', action='store_true', help="Create a .dot file for visualizing the parser via the Graphviz package.")
-	parser.add_argument('-m', '--method', choices=PARSE_TABLE_METHODS, default='LR1', type=str.upper, help="Which parser table construction method to use.")
 	parser.add_argument('-v', '--verbose', action='store_true', help="Squawk, mainly about the table compression stats.")
 	# if len(sys.argv) < 2: exit(parser.print_help())
 	return parser.parse_args()
@@ -39,16 +37,16 @@ def main(args):
 		exit(1)
 	with(open(args.source_path)) as fh:document = fh.read()
 	try:
-		intermediate_form = compile_string(document, method=args.method)
+		intermediate_form = compile_string(document)
 		if args.dot: intermediate_form.make_dot_file(target_path+'.dot')
 		textbook_form = intermediate_form.determinize()
 	except DefinitionError as e:
 		print(e.args[0], file=sys.stderr)
 		exit(1)
 	else:
-		compact = textbook_form.as_compact_form(filename=os.path.basename(args.source_path))
 		if args.pretty: textbook_form.pretty_print()
 		if args.csv: textbook_form.make_csv(target_path)
+		compact = textbook_form.as_compact_form(filename=os.path.basename(args.source_path))
 		if args.dev:
 			compact_goto = compact['parser']['goto']
 			for listname in ['row_index', 'col_index', 'quotient']:
