@@ -11,19 +11,29 @@ file begins with definitions of some semantic objects. Next come a grammar and s
 class definition for a grammar object which supplies the necessary bits to make
 the extensions over BNF work properly.
 """
-
+import string
+import sys
 from typing import List
-from ..parsing.interface import ERROR_SYMBOL
-from ..support import failureprone
+from ..parsing.interface import ERROR_SYMBOL, SemanticError
 from ..scanning import miniscan
 from ..scanning.interface import ScannerBlocked
 from ..parsing import context_free, miniparse
 from ..parsing.interface import ParseError
+from ..parsing.all_methods import PARSE_TABLE_METHODS
+from ..support.failureprone import illustration
 
 NONDET = object()
+METHOD = object()
 VOID = object()
+VOID_SET = object()
 
-class DefinitionError(Exception): pass
+SET_PATTERNS = {
+	"upper" : lambda x: x.isalpha() and x.isupper(),
+	"lower" : lambda x: x.isalpha() and x.islower(),
+	"punct" : lambda x: all(c in string.punctuation for c in x),
+}
+
+class DefinitionError(SemanticError): pass
 
 ### Semantic categories for the breakdown of rewrite rules:
 class Action:
@@ -52,7 +62,7 @@ class Symbol(Element):
 	
 	def is_void_for(self, ebnf: "EBNF_Definition", head: str) -> bool:
 		key = head if self.name == '_' else self.name
-		return key in ebnf.void_symbols
+		return ebnf.is_symbol_void(key)
 
 
 class InlineRenaming(Element):
@@ -82,7 +92,6 @@ class MacroCall(Element):
 		self.name, self.actual_parameters = name, tuple(actual_parameters)
 		assert isinstance(self.name, str)
 		for a in self.actual_parameters: assert isinstance(a, Element), type(a)
-	def canonical_symbol(self, bindings:dict) -> str: return
 	
 	def implement(self, ebnf: "EBNF_Definition", head: str, bindings: dict) -> str:
 		args = [a.implement(ebnf, head, bindings) for a in self.actual_parameters]
@@ -91,7 +100,7 @@ class MacroCall(Element):
 		return symbol
 	
 	def is_void_for(self, ebnf: "EBNF_Definition", head: str) -> bool:
-		return self.name in ebnf.void_symbols
+		return ebnf.is_symbol_void(self.name)
 
 
 class Rewrite:
@@ -122,7 +131,7 @@ class Rewrite:
 			if isinstance(elt, Action):
 				placeholder = ':'+elt.name
 				raw_bnf.append(placeholder)
-				ebnf.internal_action(placeholder, prefix_capture(args, i))
+				ebnf.internal_action(placeholder, prefix_capture(args, i), self.line_nr)
 			else:
 				assert isinstance(elt, Element)
 				raw_bnf.append(elt.implement(ebnf, head, bindings))
@@ -205,12 +214,20 @@ METAGRAMMAR.rule('symbol', 'name ( ' + list_of('actual_parameter', ',') + ' )')(
 NAMES = one_or_more('name')
 METAGRAMMAR.rule('precedence', 'associativity '+one_or_more('terminal'))(None)
 METAGRAMMAR.rule('precedence', 'pragma_nondeterministic')(lambda : (NONDET, ()))
+@METAGRAMMAR.rule('precedence', 'pragma_method name')
+def parse_method(_, method):
+	try:
+		return METHOD, PARSE_TABLE_METHODS[method]
+	except KeyError:
+		error_message = "Unrecognized parse method %r. Options are %r." % (method, list(PARSE_TABLE_METHODS))
+		raise DefinitionError(error_message)
 
-METAGRAMMAR.rule('associativity', 'pragma_left')(lambda x: context_free.LEFT)
-METAGRAMMAR.rule('associativity', 'pragma_right')(lambda x: context_free.RIGHT)
-METAGRAMMAR.rule('associativity', 'pragma_nonassoc')(lambda x: context_free.NONASSOC)
-METAGRAMMAR.rule('associativity', 'pragma_bogus')(lambda x: context_free.BOGUS)
-METAGRAMMAR.rule('associativity', 'pragma_void')(lambda x: VOID)
+METAGRAMMAR.rule('associativity', 'pragma_left')(lambda _: context_free.LEFT)
+METAGRAMMAR.rule('associativity', 'pragma_right')(lambda _: context_free.RIGHT)
+METAGRAMMAR.rule('associativity', 'pragma_nonassoc')(lambda _: context_free.NONASSOC)
+METAGRAMMAR.rule('associativity', 'pragma_bogus')(lambda _: context_free.BOGUS)
+METAGRAMMAR.rule('associativity', 'pragma_void')(lambda _: VOID)
+METAGRAMMAR.rule('associativity', 'pragma_void_set')(lambda _: VOID_SET)
 
 # Sub-language: For specifying the connections between scan conditions:
 METAGRAMMAR.rule('condition', 'name')(lambda x:(x,[]))
@@ -223,7 +240,7 @@ LEX.token('name', r'\l\w*') # Identifiers as token type "name".
 LEX.token('err_token', r'\$error\$') # the error token is a bit special: it can't be an LHS or have precedence.
 LEX.token('topic', r'_/\W') # Bare underline means "the current production rule head".
 LEX.token_map('message', r':\l\w*', lambda tx:tx[1:]) # Strip out the colon for message names
-@LEX.on(r'%\l+') # Build pragma token types from the text.
+@LEX.on(r'%\l[\l_]*') # Build pragma token types from the text.
 def pragma(yy): yy.token('pragma_'+yy.match()[1:])
 LEX.token('capture', r'[.]/\S') # a dot prefixes a captured element, so it needs to be followed by something.
 @LEX.on(r'[\][(),|]') # Punctuation is represented directly, with null semantic value.
@@ -237,9 +254,13 @@ class ErrorHelper:
 	Right now I'm not sure what else to call this. Naming things is hard, after all.
 	The concept is to get the error reporting strategy defined once and accessible throughout.
 	"""
-	def __init__(self): self.current_line_nr = 0
-	def gripe(self, message): raise DefinitionError('At line %d: %s.'%(self.current_line_nr, message))
-	def gripe_about(self, line:str, column:int, message): self.gripe(message +"\n" + failureprone.illustration(line, column, prefix='\t'))
+	def __init__(self, filename:str):
+		self.current_line_nr = 0
+		self.filename = filename
+	def gripe_about(self, line:str, column:int, message):
+		print("%s -- line %d column %d: %s"%(self.filename, self.current_line_nr, column+1, message), file=sys.stderr)
+		print(illustration(line, column, prefix='\t'), file=sys.stderr)
+		raise
 	def parse(self, line:str, line_nr:int, language:str):
 		""" Factoring out the commonalities of half-decent error reporting... """
 		assert isinstance(line_nr, int), type(line_nr)
@@ -248,6 +269,8 @@ class ErrorHelper:
 		try: return METAGRAMMAR.parse(metascan, language=language)
 		except ScannerBlocked as e:
 			self.gripe_about(line, e.args[0], "The MacroParse MetaScanner got confused by %r" % e.args[1])
+		except DefinitionError as e:
+			self.gripe_about(line, metascan.left, e.args[0])
 		except ParseError:
 			self.gripe_about(
 				line, metascan.left,
@@ -273,8 +296,6 @@ class MacroDefinition:
 # Please note: The format of an action entry shall be the tuple <message_name, tuple-of-offsets, line_number>.
 # These are a pass-through into the ContextFreeGrammar and eventually the parse tables themselves.
 
-def illustrate_position(line:str, column:int): return line[:column]+'<<_here_>>'+line[column:]
-
 class EBNF_Definition:
 	def __init__(self, error_help:ErrorHelper):
 		self.plain_cfg = context_free.ContextFreeGrammar()
@@ -285,34 +306,42 @@ class EBNF_Definition:
 		self.must_elaborate = []
 		self.error_help = error_help
 		self.is_nondeterministic = False
-		self.void_symbols = set()
+		self.__void_symbols = set()
+		self.__void_sets = {self.__void_symbols.__contains__}
+		self.method = PARSE_TABLE_METHODS["LR1"]
+		self.__schedule = []
+	
+	def is_symbol_void(self, symbol:str):
+		return any(predicate(symbol) for predicate in self.__void_sets)
 	
 	def read_precedence_line(self, line:str, line_nr:int):
 		direction, symbols = self.error_help.parse(line, line_nr, 'precedence')
-		if direction is NONDET: self.is_nondeterministic = True
-		elif direction is VOID: self.void_symbols.update(symbols)
+		if direction is NONDET:
+			self.is_nondeterministic = True
+			self.method = PARSE_TABLE_METHODS["LALR"]
+		elif direction is METHOD: self.method = symbols
+		elif direction is VOID: self.__void_symbols.update(symbols)
+		elif direction is VOID_SET:
+			try: self.__void_sets.update(SET_PATTERNS[k.lower()] for k in symbols)
+			except KeyError: self.error_help.gripe_about(line, 10, "The only valid void_set patterns are %r"%(list(SET_PATTERNS)))
 		else: self.plain_cfg.assoc(direction, symbols)
 
 	def read_production_line(self, line:str, line_nr:int):
 		""" This is the main interface to defining grammars. Call this repeatedly for each line of the grammar. """
-		production = self.error_help.parse(line, line_nr, 'production')
-		if production is None:
-			self.error_help.gripe("Yeah, I'm not sure what happened there but it gave me indigestion.")
-			return
-		head, rewrites = production
+		head, rewrites = self.error_help.parse(line, line_nr, 'production')
 		for R in rewrites: R.line_nr = line_nr
 		# Set the current head field, or use it unchanged if not specified on this line:
 		if head is None:
 			if self.current_head is None:
-				self.error_help.gripe('Confused about what the current head nonterminal is')
+				self.error_help.gripe_about(line, 1, 'Confused about what the current head nonterminal is')
 			head = self.current_head
 		else:
 			if isinstance(head, tuple): # Do we need to enter a macro declaration?
 				name, formals = head
 				if name in self.macro_definitions: # Prevent re-declarations.
-					self.error_help.gripe("Cannot re-declare macro %r, which was orginally declared on line %d."%(name, self.macro_definitions[name].line_nr))
+					self.error_help.gripe_about(line, 1, "Cannot re-declare macro %r, which was orginally declared on line %d."%(name, self.macro_definitions[name].line_nr))
 				elif len(set(formals)|{name}) <= len(formals):
-					self.error_help.gripe("All the names used in a macro head declaration must be distinct.")
+					self.error_help.gripe_about(line, 1, "All the names used in a macro head declaration must be distinct.")
 				else:
 					head = self.macro_definitions[name] = MacroDefinition(name, formals)
 			self.current_head = head
@@ -323,10 +352,10 @@ class EBNF_Definition:
 			assert isinstance(head, str)
 			if self.inferential_start is None: self.inferential_start = head
 			for R in rewrites:
-				R.install(self, head, {})
+				self.__schedule.append((head, R))
 		pass
 	
-	def validate(self):
+	def __validate(self):
 		unused_macros = sorted(name+' at line '+str(definition.rewrites[0].line_nr) for name, definition in self.macro_definitions.items() if not definition.actually_used)
 		if unused_macros: raise DefinitionError('The following macro(s) were defined but never used:\n\t'+'\n\t'.join(unused_macros))
 		if not self.inferential_start: raise DefinitionError("No production rules have been given, so how am I to compile a grammar? (You could give a trivial one...)")
@@ -337,15 +366,19 @@ class EBNF_Definition:
 	
 	def sugarless_form(self) -> context_free.ContextFreeGrammar:
 		if self.inferential_start: # In other words, if any rules were ever given...
+			for head,rewrite in self.__schedule:
+				rewrite.install(self, head, {})
+				
 			while self.must_elaborate:
 				(symbol, name, args) = self.must_elaborate.pop()
 				try: definition = self.macro_definitions[name]
 				except KeyError: raise DefinitionError("At line %d macro \"%s\" is called but never defined."%(self.implementations[symbol], name))
 				else: definition.elaborate(self, symbol, args)
-			self.validate()
+				
+			self.__validate()
 			return self.plain_cfg
 	
-	def internal_action(self, placeholder:str, capture:tuple):
+	def internal_action(self, placeholder:str, capture:tuple, line_nr):
 		""" These are implemented internally much like an epsilon rule, but able to capture certain left-context. """
 		if self.implementing(placeholder):
 			self.plain_cfg.add_rule(context_free.Rule(
@@ -353,13 +386,11 @@ class EBNF_Definition:
 				rhs = (),
 				prec_sym = None,
 				action = context_free.SemanticAction(placeholder, capture),
-				provenance = self.error_help.current_line_nr,
+				provenance = line_nr,
 			))
 		else:
-			self.error_help.gripe(
-				'Internal action %s was first elaborated on line %d; reuse is not (presently) supported.' %
-				(placeholder, self.implementations[placeholder])
-			)
+			error_message = 'Internal action %s was first elaborated on line %d; reuse is not supported.' % (placeholder, self.implementations[placeholder])
+			raise DefinitionError(error_message)
 
 	def implementing(self, symbol) -> bool:
 		""" Tells if the current instantiation of a macro-rule is the first; notes the line number where it happened. """
