@@ -1,5 +1,5 @@
 
-from typing import Optional, NamedTuple, Sequence, Union
+from typing import Optional, NamedTuple, Sequence
 
 class ScanAction(NamedTuple):
 	""" The information necessary to connect to a driver (presumably) or yield a usable error message. """
