@@ -1,8 +1,14 @@
 """ Bits and bobs in support of visualizing data structures. """
 import csv
+from itertools import zip_longest
 
 DOT = '\u25cf'
 
+def dotted(rhs, position):
+	items = [str(s) for s in rhs]
+	items.insert(position, DOT)
+	return "[ %s ]"%" ".join(items)
+
 
 def print_grid(grid):
 	lens = list(map(len, grid))
@@ -26,6 +32,10 @@ def print_grid(grid):
 		if r %5 == 1: print(divider)
 		print(vertical.join(s.rjust(w,' ') for s,w in zip(row, width)))
 	print(lower.join(segments))
+	
+def print_columnar(*columns):
+	margin = range(max(map(len, columns)))
+	print_grid(list(zip_longest(margin, *columns)))
 
 def write_csv_grid(path, grid):
 	with open(path, 'w', newline="") as fh:
